#!/bin/bash
# $Id: email_delay_tracking.sh 568 2006-06-01 07:58:38Z patpro $
#
# 1.5		2006-05-30		switch data storage to sqlite
# 1.1		2006-05-28		ajout graph zoom.
# 1.0		2006-05-22		version initiale : Patrick Proniewski
#
# Script designed to send email at regular interval, and to keep track of
# their delay until they come back.
# Works along with filtrage.awk to keep track of delays added by each "hop" 
# during email delivery process
#
# Usage: the script is launched by crontab every X seconds (X>100). It creates
# an email message on MY_SMTP via an ssh connection and log the date on two
# files. Then it updates data for expected emails and plots data for received 
# emails.
# When fetchmail gets an email from the pop server, it's handled by a .forward
# file. This file pushes the email through a pipe into the awk scripts.
#
# some settings 
MY_SMTP="MY_SMTP.DOMAINE"                   # smtp server to start from
MY_ROOT="/Chemin/Des/Fichiers"              # working directory
MY_SSHKEY="/Chemin/De/La/Cle/SSH"           # ssh key to create email on MY_SMTP
MY_DATEDB="MY_DATEDB.db"                    # sqlite3 database to store datetime/delays records
MY_SQLITE="/usr/bin/sqlite3"                # path to sqlite binary
MY_GRAPH="MY_GRAPH.png"                     # gnuplot output
MY_GRAPH_ZOOM="MY_GRAPH_ZOOM.png"           # gnuplot output
MY_LOGFILE="MY_LOGFILE.log"                 # flat text file to log info
MY_RANGE=$((30*60*60))                      # range of data plotted 
MY_TZ=$((2*60*60))                          # shift from GMT for gnuplot, in seconds
MY_ACCOUNT="MY_ACCOUNT@DOMAINE"             # pop account to retrieve email from
MY_FLAG="E"                                 # Flag used for expected emails
#
LV1=60                                      # level for alarm, the nbr corresponds to the step
LV2=60                                      # in the mail delivery
LV3=60                                      # alarms are not yet triggered 
LV4=60

# create & move to MY_ROOT sandbox or die
[ -d $MY_ROOT ] || mkdir -p "${MY_ROOT}"
cd "${MY_ROOT}" || exit 1

# move stderr into MY_LOGFILE
exec 6>&2 # backup stderr into file descriptor 6
exec 2>${MY_LOGFILE} # replace stderr with MY_LOGFILE
# enable debuging
set -o xtrace

createdb() {
${MY_SQLITE} "${MY_DATEDB}" <<EOF
CREATE TABLE mailin(dateunix INTEGER primarykey, flag varchar(25), d1 INTEGER, d2 INTEGER, d3 INTEGER, d4 INTEGER);
CREATE TABLE mailout(dateunix INTEGER, d1 INTEGER);
EOF
}
[ -f $MY_DATEDB ] || createdb

# From now, every file path is relative do MY_ROOT

# a basic function to log informations into a text file
function logrun() {
	echo -n $(date) >&2
	echo " # $*" >&2
}

# trigger mail creation on MY_SMTP and feed data into MY_DATEDB
function init_mail() {
	# the ssh request create email and retrieve date in seconds from epoch
	REMOTE_DATE=$(ssh -i ${MY_SSHKEY} ${MY_SMTP} "echo test | mail ${MY_ACCOUNT} & date +%s")
	LOCAL_DATE=$(date +%s)
	# if REMOTE_DATE looks good, then proceed.
	if [ ${REMOTE_DATE:=0} -gt $((LOCAL_DATE-3600)) ]; then
		${MY_SQLITE} -line "${MY_DATEDB}" 'INSERT INTO mailin VALUES('${REMOTE_DATE}',"'${MY_FLAG}'",0,0,0,0);'
		${MY_SQLITE} -line "${MY_DATEDB}" 'INSERT INTO mailout VALUES('${REMOTE_DATE}','$((LOCAL_DATE-REMOTE_DATE))');'
	else
		logrun "init_mail failed, REMOTE_DATE doesn't look good (${REMOTE_DATE})"
	fi
}

function update_datelist() {
	STAMP=$(echo $POOL | awk '{print $1}') 
	# attention: the ${STAMP:0:8} grep makes it very risky to use a delay lesser than 100 seconds between each email
	echo $POOL | awk '
		BEGIN {A=""} {
			DU=$1;D1=$6;D2=$5;D3=$4;D4=$3;
			if($6>'$LV1') {A=A"+FLAG1"} 
			if($5>'$LV2') {A=A"+FLAG2"}
			if($4>'$LV3') {A=A"+FLAG3"}
			if($3>'$LV4') {A=A"+FLAG4"}
		} END {print "UPDATE mailin SET dateunix="DU",flag=\""A"\",d1="D1",d2="D2",d3="D3",d4="D4" WHERE dateunix LIKE \"'${STAMP:0:8}'%\"; "}' | ${MY_SQLITE} "${MY_DATEDB}"
	# we remove the POPed mail from the expected email list:
	${MY_SQLITE} -line "${MY_DATEDB}" 'DELETE FROM mailout WHERE dateunix LIKE "'${STAMP:0:8}'%";'
}

function update_expected() {
	# then we update other "expected email" records:
	LOCAL_DATE=$(date +%s)
	${MY_SQLITE} -line "${MY_DATEDB}" 'UPDATE mailout SET d1=('$LOCAL_DATE'-dateunix);'
}

function graph_datelist() {
	# last date value of MY_DATEDB.mailin, used to compute xrange
	LAST_DATE=$(${MY_SQLITE} "${MY_DATEDB}" 'SELECT max(dateunix) FROM mailin;')
	FIRST_DATE=$((LAST_DATE-MY_RANGE))
	# max value of MY_DATEDB.mailin in the MY_RANGE window, used to compute yrange
	MAX_DELAY=$(${MY_SQLITE} "${MY_DATEDB}" 'SELECT max(d1+d2+d3+d4) FROM mailin WHERE dateunix>='${FIRST_DATE}';')
	MAX_DELAY_ZOOM=$(${MY_SQLITE} "${MY_DATEDB}" 'SELECT max(d1+d2+d3+d4) FROM mailin WHERE dateunix>='$((LAST_DATE-7200))';')
	if [ $MAX_DELAY_ZOOM -gt 200 ]; then
		MAX_DELAY_ZOOM=200
	fi
	
	/opt/local/bin/gnuplot <<EOF
set terminal png size 1024,600
set output "${MY_GRAPH}"
set format xy "%g"
set xdata time
set timefmt "%s"
set format x "%H"
set ylabel "dlais total en secondes"
set yrange [0:$((MAX_DELAY+MAX_DELAY*5/100))]
set xrange [$((FIRST_DATE+MY_TZ)):$((LAST_DATE+MY_TZ))]
set xlabel "heure d'envoi (fenetre glissante de $((MY_RANGE/3600)) heures)"
set style data boxes
set style fill solid 0.70 noborder
set title "Dlais aux diffrentes tapes de l'acheminement des courriers"
plot "< ${MY_SQLITE} -separator ' ' -nullvalue '0' ${MY_DATEDB} 'SELECT dateunix FROM mailin WHERE dateunix>=$FIRST_DATE ;' | /opt/local/bin/gawk '{ if(substr(strftime(\"%H:%M\",\$1),0,4)>=\"22:0\" || substr(strftime(\"%H:%M\",\$1),0,4)<=\"07:0\") {print \$1\" $((MAX_DELAY+MAX_DELAY/5))\";} else {print \$1\" 0\";} }'" using (\$1+$MY_TZ):2 lt 29 notitle fs solid 0.15, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1+d2+d3+d4 FROM mailin WHERE dateunix>=$FIRST_DATE ;'" using (\$1):2 t "STEP 4" lt 36, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1+d2+d3 FROM mailin WHERE dateunix>=$FIRST_DATE ;'" using (\$1):2 t "STEP 3" lt 1, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1+d2 FROM mailin WHERE dateunix>=$FIRST_DATE ;'" using (\$1):2 t "STEP 2" lt 73, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1 FROM mailin WHERE dateunix>=$FIRST_DATE ;'" using (\$1):2 t "STEP 1" lt 94, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT 1000000,1; SELECT dateunix+$MY_TZ,d1 FROM mailout WHERE dateunix>=$FIRST_DATE ;'" using (\$1):2:(50) t "en transit" lt 0

set terminal png size 800,600
set output "${MY_GRAPH_ZOOM}"
set format xy "%g"
set xdata time
set timefmt "%s"
set format x "%H:%M"
set ylabel "dlais total en secondes"
set yrange [0:$((MAX_DELAY_ZOOM+MAX_DELAY_ZOOM*20/100))]
set xrange [$((LAST_DATE+MY_TZ-7200)):$((LAST_DATE+MY_TZ+350))]
set xlabel "heure d'envoi (fenetre glissante de $((7200/3600)) heures)"
set style data boxes
set style fill solid 0.70 noborder
set title "Dlais aux diffrentes tapes de l'acheminement des courriers"
plot "< ${MY_SQLITE} -separator ' ' -nullvalue '0' ${MY_DATEDB} 'SELECT dateunix FROM mailin WHERE dateunix>=$((LAST_DATE-7200)) ;' | /opt/local/bin/gawk '{ if(substr(strftime(\"%H:%M\",\$1),0,4)>=\"22:0\" || substr(strftime(\"%H:%M\",\$1),0,4)<=\"07:0\") {print \$1\" $((MAX_DELAY+MAX_DELAY/5))\";} else {print \$1\" 0\";} }'" using (\$1+$MY_TZ):2 lt 29 notitle fs solid 0.15, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1+d2+d3+d4 FROM mailin WHERE dateunix>=$((LAST_DATE-7200)) ;'" using (\$1):2 t "STEP 4" lt 36, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1+d2+d3 FROM mailin WHERE dateunix>=$((LAST_DATE-7200)) ;'" using (\$1):2 t "STEP 3" lt 1, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1+d2 FROM mailin WHERE dateunix>=$((LAST_DATE-7200)) ;'" using (\$1):2 t "STEP 2" lt 73, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT dateunix+$MY_TZ,d1 FROM mailin WHERE dateunix>=$((LAST_DATE-7200)) ;'" using (\$1):2 t "STEP 1" lt 94, \
"< ${MY_SQLITE} -separator ' ' ${MY_DATEDB} 'SELECT 1000000,1; SELECT dateunix+$MY_TZ,d1 FROM mailout WHERE dateunix>=$((LAST_DATE-7200)) ;'" using (\$1):2:(200) t "en transit" lt 0

EOF

}

if [ $# -eq 0 ]; then
	# If a flag file is present we just let the script sleep to avoid race 
	# condition problem
	while [ -f TRACKING_IN_PROGRESS ]; do sleep 1; done
	touch MAILING_IN_PROGRESS
	# no args, so standard behaviour: we send a test mail
	init_mail && logrun "init_mail succeeded"
else 
	# we have some args, so update behaviour: we update both lists with data
	# provided by the awk processing script
	# If a flag file is present we just let the script sleep to avoid race 
	# condition problem
	while [ -f MAILING_IN_PROGRESS -o -f TRACKING_IN_PROGRESS ]; do sleep 1; done
	touch TRACKING_IN_PROGRESS
	POOL=$*
	update_datelist && logrun "update_datelist succeeded"
fi

update_expected && logrun "update_expected succeeded"
graph_datelist && logrun "graph_datelist succeeded"

rm -f MAILING_IN_PROGRESS TRACKING_IN_PROGRESS

# disable debuging
set +o xtrace
# restore stderr, close MY_LOGFILE
exec 2>&6 6>&-

# $Id: email_delay_tracking.sh 568 2006-06-01 07:58:38Z patpro $